@@ -1,6 +1,11 @@
 """For space rocks."""
 from pathlib import Path
+
 import rich
+from rich import traceback
+
+# pretty-print tracebacks with rich
+traceback.install()
 
 # rocks modules
 # rocks.plots is lazy-loaded as it is expensive
@@ -22,7 +27,7 @@ INDEX = {}
 
 # ------
 # Welcome to rocks
-__version__ = "1.4.24"
+__version__ = "1.5.7"
 
 GREETING = rf"""
                 _
