@@ -30,6 +30,7 @@ CATALOGUES = {
             "err_diameter_up",
             "err_diameter_down",
             "method",
+            "bibcode",
         ],
     },
     "astdys": {
@@ -68,7 +69,7 @@ CATALOGUES = {
     "colors": {
         "attr_name": "colors",
         "ssodnet_name": "colors",
-        "print_columns": ["color", "value", "uncertainty", "phot_sys", "shortbib"],
+        "print_columns": ["color", "value", "uncertainty", "phot_sys"],
     },
     "diamalbedo": {
         "attr_name": "diamalbedo",
@@ -81,6 +82,7 @@ CATALOGUES = {
             "err_diameter_up",
             "err_diameter_down",
             "method",
+            "bibcode",
         ],
     },
     "diameters": {
@@ -94,6 +96,7 @@ CATALOGUES = {
             "err_diameter_up",
             "err_diameter_down",
             "method",
+            "bibcode",
         ],
     },
     "families": {
@@ -140,6 +143,11 @@ CATALOGUES = {
         "ssodnet_name": "phase_function",
         "print_columns": [],
     },
+    "spins": {
+        "attr_name": "spins",
+        "ssodnet_name": "spin",
+        "print_columns": ["period", "long_", "lat", "RA0", "DEC0", "Wp", "shortbib"],
+    },
     "taxonomies": {
         "attr_name": "taxonomies",
         "ssodnet_name": "taxonomy",
@@ -152,10 +160,17 @@ CATALOGUES = {
             "shortbib",
         ],
     },
-    "thermal_properties": {
-        "attr_name": "thermal_properties",
-        "ssodnet_name": "thermal_properties",
-        "print_columns": [],
+    "thermal_inertias": {
+        "attr_name": "thermal_inertias",
+        "ssodnet_name": "thermal_inertia",
+        "print_columns": [
+            "TI",
+            "err_TI_up",
+            "err_TI_down",
+            "dsun",
+            "method",
+            "shortbib",
+        ],
     },
     "yarkovskies": {
         "attr_name": "yarkovskies",
@@ -184,12 +199,25 @@ def pretty_print(rock, catalogue, parameter):
 
     # ------
     # Create table to echo
-    if parameter in ["diamalbedo", "diameters", "albedos"]:
-        caption = (
-            "Blue: preferred diameter, yellow: preferred albedo, green: both preferred"
-        )
+    if parameter in ["diameters", "albedos"]:
+
+        if parameter == "diameters":
+            preferred = catalogue.preferred_diameter
+            caption = "Green: preferred entry"
+        elif parameter == "albedos":
+            preferred = catalogue.preferred_albedo
+            caption = "Green: preferred entry"
+        # else:
+        #     preferred = catalogue.preferred
+        #     caption = "Blue: preferred diameter, yellow: preferred albedo, green: both preferred"
+
+    elif parameter in ["masses", "taxonomies", "diamalbedo"]:
+        caption = "Green: preferred entry"
+        preferred = catalogue.preferred
+
     else:
-        caption = "Green: preferred entry" if hasattr(catalogue, "selection") else ""
+        caption = None
+        preferred = [False for _ in range(len(catalogue))]
 
     table = Table(
         header_style="bold blue",
@@ -204,21 +232,20 @@ def pretty_print(rock, catalogue, parameter):
         catalogue = catalogue.sort_values("year").reset_index()
 
     # The columns depend on the catalogue
-    columns = CATALOGUES[parameter]["print_columns"]
+    columns = [""] + CATALOGUES[parameter]["print_columns"]
 
     for c in columns:
         table.add_column(c)
 
     # Some catalogues do not have a "preferred" attribute
-    if not hasattr(catalogue, "selection"):
-        preferred = [False for _ in range(len(catalogue))]
-    else:
-        preferred = [bool(s) for s in catalogue.selection]
+    # if not hasattr(catalogue, "preferred"):
+    #     preferred = [False for _ in range(len(catalogue))]
+    # else:
 
     # Add rows to table, styling by preferred-state of entry
     for i, pref in enumerate(preferred):
 
-        if parameter in ["diamalbedo", "diameters", "albedos"]:
+        if parameter in ["diamalbedos"]:
             if pref:
                 if (
                     catalogue.preferred_albedo[i]
@@ -239,7 +266,7 @@ def pretty_print(rock, catalogue, parameter):
             style = "bold green" if pref else "white"
 
         table.add_row(
-            *[str(catalogue[c][i]) for c in columns],
+            *[str(catalogue[c][i]) if c else str(i + 1) for c in columns],
             style=style,
         )
 
@@ -297,6 +324,20 @@ def ensure_int(value):
     return [int(v) if v else None for v in value]
 
 
+def empty_str_to_none(value):
+    for i, v in enumerate(value[:]):
+        if v == "":
+            value[i] = None
+    return value
+
+
+def empty_str_to_nan(value):
+    for i, v in enumerate(value[:]):
+        if v == "":
+            value[i] = np.nan
+    return value
+
+
 # ------
 # SsODNet catalogues as pydantic model
 # https://ssp.imcce.fr/webservices/ssodnet/api/datacloud/templates/ssodnet_datacloud.sql
@@ -305,11 +346,12 @@ def ensure_int(value):
 class Collection(pydantic.BaseModel):
     """Table de definition des references des jeux de donnees de la base SsODNet.datacloud"""
 
+    link: List[str] = [""]
+    title: List[str] = [""]
+    shortbib: List[str] = [""]
+    datasetname: List[str] = [""]
     idcollection: List[int] = [None]
     resourcename: List[str] = [""]
-    datasetname: List[str] = [""]
-    title: List[str] = [""]
-    link: List[str] = [""]
 
 
 class Methods(pydantic.BaseModel):
@@ -325,6 +367,7 @@ class Methods(pydantic.BaseModel):
     title: List[str] = [""]
     bibcode: List[str] = [""]
     doi: List[str] = [""]
+    bibtex: List[str] = [""]
 
 
 class Dataset_ref(pydantic.BaseModel):
@@ -357,29 +400,29 @@ class Astorb(pydantic.BaseModel):
     id_: List[int] = pydantic.Field([None], alias="id")
     number: List[Optional[int]] = pydantic.Field([None], alias="num")
     name: List[str] = [""]
-    OrbComputer: List[str] = [""]
+    orbit_computer: List[str] = [""]
     H: List[float] = [np.nan]
     G: List[float] = [np.nan]
     B_V: List[float] = [np.nan]
-    IRAS_Diameter: List[float] = [np.nan]
-    IRAS_Class: List[str] = [""]
-    Note_1: List[int] = [None]
-    Note_2: List[int] = [None]
-    Note_3: List[int] = [None]
-    Note_4: List[int] = [None]
-    Note_5: List[int] = [None]
-    Note_6: List[int] = [None]
-    OrbArc: List[int] = [None]
-    NumberObs: List[int] = [None]
-    YY_osc: List[int] = [None]
-    MM_osc: List[int] = [None]
-    DD_osc: List[int] = [None]
-    MeanAnomaly: List[float] = [np.nan]
-    ArgPerihelion: List[float] = [np.nan]
-    LongAscNode: List[float] = [np.nan]
-    Inclination: List[float] = [np.nan]
-    Eccentricity: List[float] = [np.nan]
-    SemimajorAxis: List[float] = [np.nan]
+    IRAS_diameter: List[float] = [np.nan]
+    IRAS_class: List[str] = [""]
+    note_1: List[int] = [None]
+    note_2: List[int] = [None]
+    note_3: List[int] = [None]
+    note_4: List[int] = [None]
+    note_5: List[int] = [None]
+    note_6: List[int] = [None]
+    orbital_arc: List[int] = [None]
+    number_observation: List[int] = [None]
+    yy_osc: List[int] = [None]
+    mm_osc: List[int] = [None]
+    dd_osc: List[int] = [None]
+    mean_anomaly: List[float] = [np.nan]
+    perihelion_argument: List[float] = [np.nan]
+    node_longitude: List[float] = [np.nan]
+    inclination: List[float] = [np.nan]
+    eccentricity: List[float] = [np.nan]
+    semi_major_axis: List[float] = [np.nan]
     YY_calulation: List[int] = [None]
     MM_calulation: List[int] = [None]
     DD_calulation: List[int] = [None]
@@ -400,15 +443,15 @@ class Astorb(pydantic.BaseModel):
     GGPEU_yy: List[int] = [None]
     GGPEU_mm: List[int] = [None]
     GGPEU_dd: List[int] = [None]
-    JD_osc: List[float] = [np.nan]
+    jd_osc: List[float] = [np.nan]
     px: List[float] = [np.nan]
     py: List[float] = [np.nan]
     pz: List[float] = [np.nan]
     vx: List[float] = [np.nan]
     vy: List[float] = [np.nan]
     vz: List[float] = [np.nan]
-    MeanMotion: List[float] = [np.nan]
-    OrbPeriod: List[float] = [np.nan]
+    mean_motion: List[float] = [np.nan]
+    orbital_period: List[float] = [np.nan]
     iddataset: List[int] = [None]
 
 
@@ -422,24 +465,24 @@ class Mpcorb(pydantic.BaseModel):
     H: List[float] = [np.nan]
     G: List[float] = [np.nan]
     ref_date: List[str] = [""]
-    MeanAnomaly: List[float] = [np.nan]
-    ArgPerihelion: List[float] = [np.nan]
-    LongAscNode: List[float] = [np.nan]
-    Inclination: List[float] = [np.nan]
-    Eccentricity: List[float] = [np.nan]
-    MeanMotion: List[float] = [np.nan]
-    SemimajorAxis: List[float] = [np.nan]
+    mean_anomaly: List[float] = [np.nan]
+    perihelion_argument: List[float] = [np.nan]
+    node_longitude: List[float] = [np.nan]
+    inclination: List[float] = [np.nan]
+    eccentricity: List[float] = [np.nan]
+    mean_motion: List[float] = [np.nan]
+    semi_major_axis: List[float] = [np.nan]
     U: List[str] = [""]
     reference: List[str] = [""]
-    NumberObs: List[int] = [None]
-    NumberOpp: List[int] = [None]
+    number_observation: List[int] = [None]
+    number_opposition: List[int] = [None]
     start_obs: List[int] = [None]
     end_obs: List[int] = [None]
-    OrbArc: List[float] = [np.nan]
+    orbital_arc: List[float] = [np.nan]
     rms: List[float] = [np.nan]
     coarse_indic: List[str] = [""]
     precise_indic: List[str] = [""]
-    OrbComputer: List[str] = [""]
+    orbit_computer: List[str] = [""]
     orbit_type: List[str] = [""]
     last_obs_date: List[str] = [""]
     iddataset: List[int] = [None]
@@ -470,17 +513,17 @@ class Cometpro(pydantic.BaseModel):
     fngA2: List[float] = [np.nan]
     fngA3: List[float] = [np.nan]
     tau: List[float] = [np.nan]
-    PerihelionDist: List[float] = [np.nan]
-    Eccentricity: List[float] = [np.nan]
-    ArgPerihelion: List[float] = [np.nan]
-    LongAscNode: List[float] = [np.nan]
-    Inclination: List[float] = [np.nan]
-    magH1: List[float] = [np.nan]
-    magR1: List[float] = [np.nan]
-    magD1: List[float] = [np.nan]
-    magH2: List[float] = [np.nan]
-    magR2: List[float] = [np.nan]
-    magD2: List[float] = [np.nan]
+    perihelion_distance: List[float] = [np.nan]
+    eccentricity: List[float] = [np.nan]
+    perihelion_argument: List[float] = [np.nan]
+    node_longitude: List[float] = [np.nan]
+    inclination: List[float] = [np.nan]
+    mag_H1: List[float] = [np.nan]
+    mag_R1: List[float] = [np.nan]
+    mag_D1: List[float] = [np.nan]
+    mag_H2: List[float] = [np.nan]
+    mag_R2: List[float] = [np.nan]
+    mag_D2: List[float] = [np.nan]
     iddataset: List[int] = [None]
 
 
@@ -592,7 +635,7 @@ class Binarymp(pydantic.BaseModel):
     iddataset: List[int] = [None]
 
 
-class Diamalbedo(pydantic.BaseModel):
+class Diamalbedo(Collection):
     """Diameters and Albedos database from literature"""
 
     number: List[Optional[int]] = pydantic.Field([None], alias="num")
@@ -609,6 +652,7 @@ class Diamalbedo(pydantic.BaseModel):
     err_emissivity: List[float] = [np.nan]
     selection: List[int] = [None]
     method: List[str] = [""]
+    bibcode: List[str] = [""]
 
     preferred_albedo: List[bool] = [False]
     preferred_diameter: List[bool] = [False]
@@ -632,15 +676,15 @@ class Diamalbedo(pydantic.BaseModel):
         ]
 
 
-class Masses(pydantic.BaseModel):
+class Masses(Collection):
     """Mass database from literature"""
 
     id_: List[int] = pydantic.Field([None], alias="id")
     number: List[Optional[int]] = pydantic.Field([None], alias="num")
     name: List[str] = [""]
     mass: List[float] = [np.nan]
-    err_mass_max: List[float] = [np.nan]
-    err_mass_min: List[float] = [np.nan]
+    err_mass_up: List[float] = [np.nan]
+    err_mass_down: List[float] = [np.nan]
     method: List[str] = [""]
     selection: List[int] = [None]
     iddataset: List[int] = [None]
@@ -652,7 +696,7 @@ class Masses(pydantic.BaseModel):
         return rocks.definitions.rank_properties("mass", values)
 
 
-class Taxonomies(pydantic.BaseModel):
+class Taxonomies(Collection):
     """Taxonomy database from literature"""
 
     # id_: List[int] = pydantic.Field([None], alias="id")
@@ -668,36 +712,41 @@ class Taxonomies(pydantic.BaseModel):
     waverange: List[str] = [""]
     # iddataset: List[int] = [None]
 
+    preferred: List[bool] = [False]
 
-class Proper_elements(pydantic.BaseModel):
+    @pydantic.validator("preferred", pre=True)
+    def select_preferred_class(cls, _, values):
+        return rocks.definitions.rank_properties("taxonomy", values)
+
+
+class Proper_elements(Collection):
     """Proper Elements from literature"""
 
     id_: List[int] = pydantic.Field([None], alias="id")
     number: List[Optional[int]] = pydantic.Field([None], alias="num")
     name: List[str] = [""]
     H: List[float] = [np.nan]
-    ProperSemimajorAxis: List[float] = [np.nan]
-    err_ProperSemimajorAxis: List[float] = [np.nan]
-    ProperEccentricity: List[float] = [np.nan]
-    err_ProperEccentricity: List[float] = [np.nan]
-    ProperSinI: List[float] = [np.nan]
-    err_ProperSinI: List[float] = [np.nan]
-    ProperInclination: List[float] = [np.nan]
-    err_ProperInclination: List[float] = [np.nan]
-    n: List[float] = [np.nan]
-    err_n: List[float] = [np.nan]
-    g: List[float] = [np.nan]
-    err_g: List[float] = [np.nan]
-    s: List[float] = [np.nan]
-    err_s: List[float] = [np.nan]
-    LCE: List[float] = [np.nan]
-    My: List[float] = [np.nan]
-    lam_fit: List[float] = pydantic.Field([np.nan], alias="lam-fit")
+    proper_semi_major_axis: List[float] = [np.nan]
+    err_proper_semi_major_axis: List[float] = [np.nan]
+    proper_eccentricity: List[float] = [np.nan]
+    err_proper_eccentricity: List[float] = [np.nan]
+    proper_sine_inclination: List[float] = [np.nan]
+    err_proper_sine_inclination: List[float] = [np.nan]
+    proper_inclination: List[float] = [np.nan]
+    err_proper_inclination: List[float] = [np.nan]
+    proper_frequency_mean_motion: List[float] = [np.nan]
+    err_proper_frequency_mean_motion: List[float] = [np.nan]
+    proper_frequency_perihelion_longitude: List[float] = [np.nan]
+    err_proper_frequency_perihelion_longitude: List[float] = [np.nan]
+    proper_frequency_nodal_longitude: List[float] = [np.nan]
+    err_proper_frequency_nodal_longitude: List[float] = [np.nan]
+    lyapunov_time: List[float] = [np.nan]
+    integration_time: List[float] = [np.nan]
     identfrom: List[str] = [""]
     iddataset: List[int] = [None]
 
 
-class Phase_function(pydantic.BaseModel):
+class Phase_function(Collection):
     """Database of asteroid phase function"""
 
     id_: List[int] = pydantic.Field([None], alias="id")
@@ -706,12 +755,12 @@ class Phase_function(pydantic.BaseModel):
     H: List[float] = [np.nan]
     G1: List[float] = [np.nan]
     G2: List[float] = [np.nan]
-    H_min: List[float] = [np.nan]
-    H_max: List[float] = [np.nan]
-    G1_min: List[float] = [np.nan]
-    G1_max: List[float] = [np.nan]
-    G2_min: List[float] = [np.nan]
-    G2_max: List[float] = [np.nan]
+    err_H_down: List[float] = [np.nan]
+    err_H_up: List[float] = [np.nan]
+    err_G1_down: List[float] = [np.nan]
+    err_G1_up: List[float] = [np.nan]
+    err_G2_down: List[float] = [np.nan]
+    err_G2_up: List[float] = [np.nan]
     N: List[int] = [None]
     phase_min: List[float] = [np.nan]
     phase_max: List[float] = [np.nan]
@@ -723,21 +772,22 @@ class Phase_function(pydantic.BaseModel):
     iddataset: List[int] = [None]
 
 
-class Families(pydantic.BaseModel):
+class Families(Collection):
     """Database of Sso families"""
 
     id_: List[int] = pydantic.Field([None], alias="id")
     number: List[Optional[int]] = pydantic.Field([None], alias="num")
     name: List[str] = [""]
     family_status: List[str] = [""]
-    family_number: List[int] = [None]
+    family_number: List[int] = pydantic.Field([None], alias="family_num")
     family_name: List[str] = [""]
     membership: List[int] = [None]
     selection: List[int] = [None]
+    method: List[str] = [""]
     iddataset: List[int] = [None]
 
 
-class Pairs(pydantic.BaseModel):
+class Pairs(Collection):
     """Database of Sso pairs"""
 
     id_: List[int] = pydantic.Field([None], alias="id")
@@ -745,17 +795,16 @@ class Pairs(pydantic.BaseModel):
     name: List[str] = [""]
     sibling_number: List[int] = pydantic.Field([None], alias="sibling_num")
     sibling_name: List[str] = [""]
-    delta_v: List[float] = [np.nan]
-    delta_a: List[float] = [np.nan]
-    delta_e: List[float] = [np.nan]
-    delta_sini: List[float] = [np.nan]
-    delta_i: List[float] = [np.nan]
-    membership: List[int] = [None]
+    distance: List[float] = [np.nan]
+    age: List[float] = [np.nan]
+    err_age_up: List[float] = [np.nan]
+    err_age_down: List[float] = [np.nan]
     selection: List[int] = [None]
+    method: List[str] = [""]
     iddataset: List[int] = [None]
 
 
-class Spin(pydantic.BaseModel):
+class Spin(Collection):
     """Database of Sso spin coordiantes"""
 
     id_: List[int] = pydantic.Field([None], alias="id")
@@ -765,15 +814,17 @@ class Spin(pydantic.BaseModel):
     model_name: List[str] = [""]
     model_id: List[str] = [""]
     t0: List[float] = [np.nan]
-    RA0: List[float] = [np.nan]
-    DEC0: List[float] = [np.nan]
     W0: List[float] = [np.nan]
     Wp: List[float] = [np.nan]
+    RA0: List[float] = [np.nan]
+    DEC0: List[float] = [np.nan]
+    err_RA0: List[float] = [np.nan]
+    err_DEC0: List[float] = [np.nan]
     period: List[float] = [np.nan]
     err_period: List[float] = [np.nan]
     period_flag: List[float] = [np.nan]
     period_type: List[str] = [""]
-    long: List[float] = [np.nan]
+    long_: List[float] = pydantic.Field([np.nan], alias="long")
     lat: List[float] = [np.nan]
     err_long: List[float] = [np.nan]
     err_lat: List[float] = [np.nan]
@@ -781,8 +832,24 @@ class Spin(pydantic.BaseModel):
     method: List[str] = [""]
     iddataset: List[str] = [""]
 
+    _empty_str_to_nan: classmethod = pydantic.validator(
+        "Wp",
+        "RA0",
+        "DEC0",
+        "err_RA0",
+        "period",
+        "err_period",
+        "err_long",
+        "err_lat",
+        "err_DEC0",
+        "long_",
+        "lat",
+        allow_reuse=True,
+        pre=True,
+    )(empty_str_to_nan)
 
-class Yarkovsky(pydantic.BaseModel):
+
+class Yarkovsky(Collection):
     """Database of Sso Yarkovsky accelerations"""
 
     id_: List[int] = pydantic.Field([None], alias="id")
@@ -799,7 +866,7 @@ class Yarkovsky(pydantic.BaseModel):
     iddataset: List[int] = [None]
 
 
-class Thermal_properties(pydantic.BaseModel):
+class Thermal_inertia(Collection):
     """Database of asteroid thermal properties"""
 
     id_: List[int] = pydantic.Field([None], alias="id")
@@ -814,7 +881,7 @@ class Thermal_properties(pydantic.BaseModel):
     iddataset: List[int] = [None]
 
 
-class Colors(pydantic.BaseModel):
+class Colors(Collection):
     """Database of asteroid colors"""
 
     id_: List[int] = pydantic.Field([None], alias="id")
@@ -823,7 +890,7 @@ class Colors(pydantic.BaseModel):
     color: List[str] = [""]
     value: List[float] = [np.nan]
     uncertainty: List[float] = [np.nan]
-    from_: List[str] = pydantic.Field([""], alias="from")
+    facility: List[str] = [""]
     observer: List[str] = [""]
     epoch: List[str] = [""]
     delta_time: List[str] = [""]
@@ -835,7 +902,7 @@ class Colors(pydantic.BaseModel):
     iddataset: List[int] = [None]
 
 
-class Density(pydantic.BaseModel):
+class Density(Collection):
     """Database of asteroid density"""
 
     id_: List[int] = pydantic.Field([None], alias="id")
@@ -849,7 +916,7 @@ class Density(pydantic.BaseModel):
     iddataset: List[int] = [None]
 
 
-class Mpcatobs(pydantic.BaseModel):
+class Mpcatobs(Collection):
     """MPCAT-OBS database"""
 
     id_: List[int] = pydantic.Field([None], alias="id")
@@ -870,7 +937,27 @@ class Mpcatobs(pydantic.BaseModel):
     note3: List[str] = [""]
     note4: List[str] = [""]
     iau_code: List[str] = [""]
+    obs_long: List[float] = [np.nan]
+    obs_lat: List[float] = [np.nan]
+    obs_alt: List[float] = [np.nan]
     vgs_x: List[float] = [np.nan]
     vgs_y: List[float] = [np.nan]
     vgs_z: List[float] = [np.nan]
     iddataset: List[int] = [None]
+
+
+class Shape(Collection):
+    """Database of Sso triaxial ellipsoid and shape models"""
+
+    id_: List[int] = pydantic.Field([None], alias="id")
+    iddataset: List[int] = [None]
+    number: List[Optional[int]] = pydantic.Field([None], alias="num")
+    name: List[str] = [""]
+    model_dbid: List[str] = [""]
+    model_name: List[str] = [""]
+    model_id: List[str] = [""]
+    scaled: List[int] = [None]
+    radius_a: List[float] = [np.nan]
+    radius_b: List[float] = [np.nan]
+    radius_c: List[float] = [np.nan]
+    selected: List[int] = [None]
