@@ -24,7 +24,7 @@ def warning_on_one_line(message, category, filename, lineno, file=None, line=Non
     if "rocks/" in filename:
         return f"rocks: {message}\n"
     else:
-        return "%s:%s: %s: %s\n" % (filename, lineno, category.__name__, message)
+        return f"{filename}:{lineno}: {category.__name__}: {message}\n"
 
 
 warnings.formatwarning = warning_on_one_line
@@ -41,30 +41,31 @@ def rgetattr(obj, attr):
     return reduce(_getattr, [obj] + attr.split("."))
 
 
-def get_unit(path_unit: str) -> str:
+def get_unit(path_parameter: str) -> str:
     """Get unit from units JSON file.
 
     Parameters
     ----------
-    path_unit : str
-        Path to the parameter in the JSON tree, starting at unit and
-        separating the levels with periods.
+    path_parameter : str
+        Path to the parameter in the mappings JSON tree.
 
     Returns
     -------
     str
         The unit of the requested parameter.
     """
-    if not rocks.PATH_MAPPING["units"].is_file():
-        retrieve_json_from_ssodnet("units")
+    if not rocks.PATH_MAPPING.is_file():
+        retrieve_mappings_from_ssodnet()
 
-    with open(rocks.PATH_MAPPING["units"], "r") as units:
-        units = json.load(units)
+    with open(rocks.PATH_MAPPING, "r") as file_:
+        mappings = json.load(file_)
 
-    for key in path_unit.split("."):
-        units = units[key]
+    if "unit" in mappings[path_parameter]:
+        unit = mappings[path_parameter]["unit"]
+    else:
+        unit = ""
 
-    return units
+    return unit
 
 
 # ------
@@ -100,8 +101,8 @@ def weighted_average(catalogue, parameter):
     observable = np.array(values)[preferred]
     error = np.array(errors)[preferred]
 
-    if all([np.isnan(value) for value in values]) or all(
-        [np.isnan(error) for error in errors]
+    if all(np.isnan(value) for value in values) or all(
+        np.isnan(error) for error in errors
     ):
         warnings.warn(
             f"{catalogue.name[0]}: The values or errors of property '{parameter}' are all NaN. Average failed."
@@ -115,7 +116,7 @@ def weighted_average(catalogue, parameter):
     if len(observable) == 1:
         return (observable[0], error[0])
 
-    if any([e == 0 for e in error]):
+    if any(e == 0 for e in error):
         weights = np.ones(observable.shape)
         warnings.warn("Encountered zero in errors array. Setting all weights to 1.")
     else:
@@ -131,8 +132,8 @@ def weighted_average(catalogue, parameter):
         len(observable)
         / (len(observable) - 1)
         * (
-            sum(w * o ** 2 for w, o in zip(weights, observable)) / sum(weights)
-            - avg ** 2
+            sum(w * o**2 for w, o in zip(weights, observable)) / sum(weights)
+            - avg**2
         )
     )
     std_avg = np.sqrt(var_avg / len(observable))
@@ -144,17 +145,15 @@ def weighted_average(catalogue, parameter):
 def retrieve_mappings_from_ssodnet():
     """Retrieve the mappings JSON from SsODNet to the cache directory."""
 
-    URL = (
-        "https://ssp.imcce.fr/webservices/ssodnet/api/ssocard/mapping_aster-astorb.json"
-    )
+    URL = "https://ssp.imcce.fr/webservices/ssodnet/api/ssocard/metadata_aster.json"
 
     response = requests.get(URL)
 
     if response.ok:
-        ssoCard = response.json()
+        metadata = response.json()
 
         with open(rocks.PATH_MAPPING, "w") as file_:
-            json.dump(ssoCard, file_)
+            json.dump(metadata["mapping"], file_)
 
     else:
         warnings.warn(f"Retrieving mappings file failed with URL:\n{URL}")
@@ -187,10 +186,8 @@ def cache_inventory():
 
         # Datacloud catalogue or ssoCard?
         if any(
-            [
-                cat["ssodnet_name"] in str(file_)
-                for cat in rocks.datacloud.CATALOGUES.values()
-            ]
+            cat["ssodnet_name"] in str(file_)
+            for cat in rocks.datacloud.CATALOGUES.values()
         ):
             *ssodnet_id, catalogue = file_.stem.split("_")
             ssodnet_id = "_".join(ssodnet_id)
@@ -215,9 +212,7 @@ def cache_inventory():
             cached_cards.append(ssodnet_id)
 
     # Get cached metadata files
-    cached_meta = (
-        [rocks.PATH_MAPPING.stem] if rocks.rocks.PATH_MAPPING.is_file() else []
-    )
+    cached_meta = [rocks.PATH_MAPPING.stem] if rocks.PATH_MAPPING.is_file() else []
     return cached_cards, cached_catalogues, cached_meta
 
 
